#include <avr/io.h>
#include <avr/interrupt.h>

#include "decoder.h"

uint16_t 	Timestamps[MAX_TIMESTAMPS];
int8_t 		TimestampCount;
uint16_t 	TimestampOld;
uint8_t 	TimeStampSendFlag;
uint16_t	BitLength;
uint32_t	BitCode;

void init_decoder(void)
{
	TimestampCount = -1;

  	TCCR1B |=  _BV(CS11) | _BV(CS10) | _BV(ICNC1) | _BV(ICES1);

	TIMSK |= _BV(TICIE1); // | _BV(OCIE1A);
}


//
// Interrupt handler for Input Capture
//

ISR(TIMER1_CAPT_vect)
{
	uint16_t val = ICR1;		// read timer value
	uint16_t diff = val - TimestampOld;
	TimestampOld = val;


	TCCR1B ^= _BV(ICES1);			// toggle rising/falling edge detection

	if (TimeStampSendFlag == 0)
	{
		if (TimestampCount < 0)		// wait for preamble
		{
			if ((diff > 400) && (diff < 900) && (TCCR1B & _BV(ICES1)))
			{
				//TCNT1 = 0;
				TimestampCount = 0;
				BitCode = 0;
			}
		}

		if (TimestampCount >= 0)			// store timestamps
		{
			if (TimestampCount > 0)
			{
				if (TimestampCount & 1)		// odd count: rising edge
					BitLength = diff;
				else						// even count: falling edge
				{
					BitCode = BitCode << 1;
					BitLength += diff;
					if (BitLength > 300)
						BitCode |= 1;
				}
			}

			if (TimestampCount == 42)		// Frame complete?
				TimeStampSendFlag = 1;

			if (TimestampCount < MAX_TIMESTAMPS)
				Timestamps[TimestampCount++] = diff;

		}
	}

}


//
// Interrupt handler for Timer overflow
//

ISR(TIMER1_COMPA_vect)
{
	TimestampCount = -1;
}

