#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include <stdlib.h>
#include <util/delay.h>

#include "global.h"
#include "rf12.h"     
#include "serial.h"
#include "decoder.h"


// init
// set IOs, Timermode etc.

void init(void)
{
	// init Ports
    // set PortA as input (pushbuttons on STK500)
    //DDRA = 0xff;
	//PORTA = 0xff;

    // set PortC as output (LEDs on STK500)
    DDRC = 0xff;
	PORTC = ~0x01;

	init_uart();
	init_decoder();
}


// ExecRFMCommand
// Read or Write RFM settings

void ExecRFMCommand(void)
{
	switch (msgRcv.header.cmd)
	{
		case msgCMD_GET_RFM_REG:
			break;
		case msgCMD_SET_RFM_REG:
			rf12_trans(msgRcv.param.item2);
			break;
	}
}


// ExecSFRCommand
// Read or Write Port settings
// param1 : SFR
// param2 : newValue

void ExecSFRCommand(void)
{
	switch (msgRcv.header.cmd)
	{
		case msgCMD_GET_SFR:
			SendSFRStatus(msgRcv.param.item1);
			break;
		case msgCMD_SET_SFR:
			_SFR_IO8(msgRcv.param.item1) = msgRcv.param.item2;
			break;
	}
}


// DispatchMessage
// interpret message and call action

void DispatchMessage(void)
{
	switch (msgRcv.header.id)
	{
		case msgID_BOOTLOADER:
			wdt_enable(WDTO_15MS);
			for (;;){};
			break;

		case msgID_RFM:				// message for RFM Setting
			ExecRFMCommand();
			break;

		case msgID_SFR:				// message for SFR command (simple test)
			ExecSFRCommand();
			break;
	}
}

//
// main
//

int main(void)
{
	init();


	rf12_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf12_setfreq(RF12FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen


	// Bandwidth		LNA gain		RSSI threshold
	// 0: reserved		0:   0  dB		0: -103 dBm
	// 1: 400 kHz		1:	-6  dB		1: -97  dBm
	// 2: 340 kHz		2:	-14 dB		2: -91  dBm
	// 3: 270 kHz		3:	-20 dB		3: -85  dBm
	// 4: 200 kHz						4: -79  dBm
	// 5: 134 kHz						5: -73  dBm
	// 6:  67 kHz						6: -67  dBm
	// 7: reserved						7: -61  dBm
	rf12_setbandwidth(4, 3, 1);		// Bandbreite, Verstrkung, DRSSI threshold


	rf12_setbaud(38400);				// 9600 baud
	rf12_setpower(0, 6);			// 1mW Ausgangangsleistung, 120kHz Frequenzshift
	
	rf12_rx_on();


	ClearMessage();
	sei();								// global Interrupt enable


	for (;;)
	{
		if (rcvState == rcvState_MsgComplete)
		{

			DispatchMessage();
			ClearMessage();
		}

		if (TimeStampSendFlag)
		{
			//SendTimestamps();
			SendBitCode();
			TimestampCount = -1;
			TimeStampSendFlag = 0;
		}
	}
}

