//
// serial communication basics
// SN, 01.06.2006
//


#include <inttypes.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "serial.h"



//
// Globals
//

uint8_t		ErrorRcvLen;
uint8_t 	ErrorBcc;
tMsg		msgRcv;
tMsg		msgSend;
uint8_t		bccRcv;
uint8_t		msgSendLen;
uint8_t		send_flag;

volatile rcvState_t	rcvState;
volatile uint8_t	msgRcvLen;


//
// simple I/O Routines
//

// send single character

void uart_putchar( char c )
{
  while( (UCSRA & _BV(UDRE)) == 0 );
  UDR = c;
}

// send string

void uart_puts( char *s )
{
  while( *s )
    uart_putchar( *s++ );
}


// InitUART
// init UART settings

void init_uart(void)
{
	UBRRL = bauddivider;								//set baud rate
	UBRRH = bauddivider >> 8;
	UCSRA = _BV(U2X);									//no U2X, MPCM
	UCSRC = _BV(URSEL) | _BV(UCSZ1) | _BV(UCSZ0);		//8 Bit
	UCSRB = _BV(RXEN) | _BV(TXEN) | _BV(RXCIE);			//enable RX, TX

	ErrorBcc = 0;
	ErrorRcvLen = 0;
}


// ClearMessage
// reset state machine and prepare for next message

void ClearMessage(void)
{
	msgRcvLen = 0;
	rcvState = rcvState_WaitForSTX;
}

//
// Interrupt handler for UART
//

ISR(USART_RXC_vect)
{
	uint8_t ch;

	ch = UDR;										// read byte from data register
	bccRcv ^= ch;										// checksum

	switch (rcvState)								// state machine
	{
		case rcvState_WaitForSTX:					// wait for message start
			if (ch == STX)
			{
				bccRcv = STX;
				rcvState = rcvState_RcvData;		// ok, next state
			}
			break;

		case rcvState_RcvData:						// wait for data
			if (ch == ETX)							// received End of message?
			{
				rcvState = rcvState_WaitForBCC;
				goto retFromUART_RECV;
			}
			if (ch == DLE)							// received Escape char? for DLE, STX and ETX values
			{
				rcvState = rcvState_ReceivedDLE;
				goto retFromUART_RECV;
			}
			// no break, continue RcvData

		case rcvState_ReceivedDLE:
			rcvState = rcvState_RcvData;
			if (msgRcvLen < MAX_MSGLEN)				// max. Length for message?
			{
				msgRcv.buffer[msgRcvLen] = ch;			// ok, add to message
				msgRcvLen++;
			}
			else
			{
				ErrorRcvLen++;						// increment error count
				rcvState = rcvState_WaitForSTX;
			}
			break;

		case rcvState_WaitForBCC:					// wait for checksum
			{
				if (bccRcv == 0)
					rcvState = rcvState_MsgComplete;
				else
				{
					rcvState = rcvState_WaitForSTX;
					ErrorBcc++;
				}
			}
			break;

		case rcvState_MsgComplete:					// message must be read and state resetted from main thread
			break;
	}

retFromUART_RECV:
	ch = ch;										// GNU-C complained about label without code
}


//
// send_message
// message must be prepared in the global sendbuffer 'msgSend' and 'msgSendLen' before this call

void SendMessage(void)
{
	uint8_t bcc = STX;

	uart_putchar(STX);								// start message with STX

	for (uint8_t i=0; i<msgSendLen; i++)			// send databytes
	{
		uint8_t b = msgSend.buffer[i];				// get databyte

		if ((b == STX) || (b == ETX) || (b == DLE)) // check for controlchar in datastream
		{
			uart_putchar(DLE);						// send DLE char
			bcc ^= DLE;
		}

		uart_putchar(b);							// send databyte
		bcc ^= b;									// calc bcc
	}

	uart_putchar(ETX);								// send ETX char
	bcc ^= ETX;

	uart_putchar(bcc);								// and bcc

	send_flag = 0;
}


